import 'package:flutter/material.dart';
import 'package:font_awesome_flutter/font_awesome_flutter.dart';
import 'package:senior_design_tokens/senior_design_tokens.dart';

enum SeniorBadgeShape { pill, chip }

class SeniorBadge extends StatelessWidget {
  final Color backgroundColor;
  final Color? disabledBackgroundColor;
  final Color fontColor;
  final Color? disabledFontColor;
  final String label;
  final bool disabled;
  final SeniorBadgeShape shape;

  const SeniorBadge({
    Key? key,
    required this.backgroundColor,
    this.disabledBackgroundColor,
    required this.fontColor,
    this.disabledFontColor,
    required this.label,
    this.disabled = false,
    this.shape = SeniorBadgeShape.chip,
  }) : super(key: key);

  factory SeniorBadge.icon({
    Key? key,
    required Color backgroundColor,
    Color? disabledBackgroundColor,
    required Color fontColor,
    Color? disabledFontColor,
    required String label,
    required IconData icon,
    bool disabled,
    SeniorBadgeShape shape,
  }) = _SeniorBadgeIcon;

  Color getBackgroundColor() {
    if (disabledBackgroundColor == null) {
      return backgroundColor;
    }

    return disabled ? disabledBackgroundColor! : backgroundColor;
  }

  Color getFontColor() {
    if (disabledFontColor == null) {
      return fontColor;
    }

    return disabled ? disabledFontColor! : fontColor;
  }

  double getShapeRadius() {
    switch (shape) {
      case SeniorBadgeShape.pill:
        return SeniorRadius.huge;
      case SeniorBadgeShape.chip:
      default:
        return SeniorRadius.medium;
    }
  }

  @override
  Widget build(BuildContext context) {
    return Container(
      padding: const EdgeInsets.symmetric(
        horizontal: SeniorSpacing.normal,
        vertical: SeniorSpacing.xxsmall,
      ),
      decoration: BoxDecoration(
        color: getBackgroundColor(),
        borderRadius: BorderRadius.all(
          Radius.circular(getShapeRadius()),
        ),
      ),
      child: Text(
        label,
        style: SeniorTypography.small(
          color: getFontColor(),
        ),
      ),
    );
  }
}

class _SeniorBadgeIcon extends SeniorBadge {
  final IconData icon;

  const _SeniorBadgeIcon({
    Key? key,
    required Color backgroundColor,
    Color? disabledBackgroundColor,
    required Color fontColor,
    Color? disabledFontColor,
    required String label,
    required this.icon,
    bool disabled = false,
    SeniorBadgeShape shape = SeniorBadgeShape.pill,
  }) : super(
          key: key,
          backgroundColor: backgroundColor,
          disabledBackgroundColor: disabledBackgroundColor,
          fontColor: fontColor,
          disabledFontColor: disabledFontColor,
          label: label,
          disabled: disabled,
          shape: shape,
        );

  @override
  Widget build(BuildContext context) {
    return Container(
      padding: const EdgeInsets.symmetric(
        vertical: SeniorSpacing.xxsmall,
        horizontal: SeniorSpacing.small,
      ),
      decoration: BoxDecoration(
        color: getBackgroundColor(),
        borderRadius: BorderRadius.all(
          Radius.circular(getShapeRadius()),
        ),
      ),
      child: Row(
        mainAxisSize: MainAxisSize.min,
        crossAxisAlignment: CrossAxisAlignment.start,
        mainAxisAlignment: MainAxisAlignment.start,
        children: [
          Padding(
            padding: const EdgeInsets.only(
              right: SeniorSpacing.xsmall,
            ),
            child: FaIcon(
              icon,
              color: getFontColor(),
              size: SeniorTypographyIconSize.text,
            ),
          ),
          Text(
            label,
            style: SeniorTypography.title(
              color: getFontColor(),
            ),
          ),
        ],
      ),
    );
  }
}
